@@ -69,12 +69,19 @@ class BaseFief:
         client_secret: str,
         *,
         encryption_key: Optional[str] = None,
+        host: Optional[str] = None,
     ) -> None:
         self.base_url = base_url
         self.client_id = client_id
         self.client_secret = client_secret
         if encryption_key is not None:
             self.encryption_key = jwk.JWK.from_json(encryption_key)
+        self.host = host
+
+    def _get_endpoint_url(
+        self, openid_configuration: Dict[str, Any], field: str
+    ) -> str:
+        return openid_configuration[field]
 
     def _auth_url(
         self,
@@ -83,6 +90,8 @@ class BaseFief:
         *,
         state: str = None,
         scope: Optional[List[str]] = None,
+        code_challenge: Optional[str] = None,
+        code_challenge_method: Optional[str] = None,
         extras_params: Optional[Mapping[str, str]] = None,
     ) -> str:
         params = {
@@ -97,10 +106,16 @@ class BaseFief:
         if scope is not None:
             params["scope"] = " ".join(scope)
 
+        if code_challenge is not None and code_challenge_method is not None:
+            params["code_challenge"] = code_challenge
+            params["code_challenge_method"] = code_challenge_method
+
         if extras_params is not None:
             params = {**params, **extras_params}
 
-        authorization_endpoint = openid_configuration["authorization_endpoint"]
+        authorization_endpoint = self._get_endpoint_url(
+            openid_configuration, "authorization_endpoint"
+        )
         return f"{authorization_endpoint}?{urlencode(params)}"
 
     def _validate_access_token(
@@ -168,7 +183,13 @@ class BaseFief:
         return client.build_request("GET", "/.well-known/openid-configuration")
 
     def _get_auth_exchange_token_request(
-        self, client: HTTPXClient, *, endpoint: str, code: str, redirect_uri: str
+        self,
+        client: HTTPXClient,
+        *,
+        endpoint: str,
+        code: str,
+        redirect_uri: str,
+        code_verifier: Optional[str] = None,
     ) -> httpx.Request:
         basic_auth = httpx.BasicAuth(self.client_id, self.client_secret)
         return client.build_request(
@@ -181,6 +202,7 @@ class BaseFief:
                 "grant_type": "authorization_code",
                 "code": code,
                 "redirect_uri": redirect_uri,
+                "code_verifier": code_verifier,
             },
         )
 
@@ -224,6 +246,8 @@ class Fief(BaseFief):
         *,
         state: str = None,
         scope: Optional[List[str]] = None,
+        code_challenge: Optional[str] = None,
+        code_challenge_method: Optional[str] = None,
         extras_params: Optional[Mapping[str, str]] = None,
     ) -> str:
         openid_configuration = self._get_openid_configuration()
@@ -232,13 +256,17 @@ class Fief(BaseFief):
             redirect_uri,
             state=state,
             scope=scope,
+            code_challenge=code_challenge,
+            code_challenge_method=code_challenge_method,
             extras_params=extras_params,
         )
 
     def auth_callback(
-        self, code: str, redirect_uri: str
+        self, code: str, redirect_uri: str, *, code_verifier: Optional[str] = None
     ) -> Tuple[FiefTokenResponse, Dict[str, Any]]:
-        token_response = self._auth_exchange_token(code, redirect_uri)
+        token_response = self._auth_exchange_token(
+            code, redirect_uri, code_verifier=code_verifier
+        )
         jwks = self._get_jwks()
         userinfo = self._decode_id_token(
             token_response["id_token"],
@@ -251,7 +279,9 @@ class Fief(BaseFief):
     def auth_refresh_token(
         self, refresh_token: str, *, scope: Optional[List[str]] = None
     ) -> Tuple[FiefTokenResponse, Dict[str, Any]]:
-        token_endpoint = self._get_openid_configuration()["token_endpoint"]
+        token_endpoint = self._get_endpoint_url(
+            self._get_openid_configuration(), "token_endpoint"
+        )
         with self._get_httpx_client() as client:
             request = self._get_auth_refresh_token_request(
                 client,
@@ -281,7 +311,9 @@ class Fief(BaseFief):
         )
 
     def userinfo(self, access_token: str) -> Dict[str, Any]:
-        userinfo_endpoint = self._get_openid_configuration()["userinfo_endpoint"]
+        userinfo_endpoint = self._get_endpoint_url(
+            self._get_openid_configuration(), "userinfo_endpoint"
+        )
         with self._get_httpx_client() as client:
             request = self._get_userinfo_request(
                 client, endpoint=userinfo_endpoint, access_token=access_token
@@ -298,7 +330,11 @@ class Fief(BaseFief):
 
     @contextlib.contextmanager
     def _get_httpx_client(self):
-        with httpx.Client(base_url=self.base_url) as client:
+        headers = {}
+        if self.host is not None:
+            headers["Host"] = self.host
+
+        with httpx.Client(base_url=self.base_url, headers=headers) as client:
             yield client
 
     def _get_openid_configuration(self) -> Dict[str, Any]:
@@ -316,20 +352,25 @@ class Fief(BaseFief):
         if self._jwks is not None:
             return self._jwks
 
-        jwks_uri = self._get_openid_configuration()["jwks_uri"]
+        jwks_uri = self._get_endpoint_url(self._get_openid_configuration(), "jwks_uri")
         with self._get_httpx_client() as client:
             response = client.get(jwks_uri)
             self._jwks = jwk.JWKSet.from_json(response.text)
             return self._jwks
 
-    def _auth_exchange_token(self, code: str, redirect_uri) -> FiefTokenResponse:
-        token_endpoint = self._get_openid_configuration()["token_endpoint"]
+    def _auth_exchange_token(
+        self, code: str, redirect_uri: str, *, code_verifier: Optional[str] = None
+    ) -> FiefTokenResponse:
+        token_endpoint = self._get_endpoint_url(
+            self._get_openid_configuration(), "token_endpoint"
+        )
         with self._get_httpx_client() as client:
             request = self._get_auth_exchange_token_request(
                 client,
                 endpoint=token_endpoint,
                 code=code,
                 redirect_uri=redirect_uri,
+                code_verifier=code_verifier,
             )
             response = client.send(request)
 
@@ -345,6 +386,8 @@ class FiefAsync(BaseFief):
         *,
         state: str = None,
         scope: Optional[List[str]] = None,
+        code_challenge: Optional[str] = None,
+        code_challenge_method: Optional[str] = None,
         extras_params: Optional[Mapping[str, str]] = None,
     ) -> str:
         openid_configuration = await self._get_openid_configuration()
@@ -353,13 +396,17 @@ class FiefAsync(BaseFief):
             redirect_uri,
             state=state,
             scope=scope,
+            code_challenge=code_challenge,
+            code_challenge_method=code_challenge_method,
             extras_params=extras_params,
         )
 
     async def auth_callback(
-        self, code: str, redirect_uri: str
+        self, code: str, redirect_uri: str, *, code_verifier: Optional[str] = None
     ) -> Tuple[FiefTokenResponse, Dict[str, Any]]:
-        token_response = await self._auth_exchange_token(code, redirect_uri)
+        token_response = await self._auth_exchange_token(
+            code, redirect_uri, code_verifier=code_verifier
+        )
         jwks = await self._get_jwks()
         userinfo = self._decode_id_token(
             token_response["id_token"],
@@ -372,7 +419,9 @@ class FiefAsync(BaseFief):
     async def auth_refresh_token(
         self, refresh_token: str, *, scope: Optional[List[str]] = None
     ) -> Tuple[FiefTokenResponse, Dict[str, Any]]:
-        token_endpoint = (await self._get_openid_configuration())["token_endpoint"]
+        token_endpoint = self._get_endpoint_url(
+            await self._get_openid_configuration(), "token_endpoint"
+        )
         async with self._get_httpx_client() as client:
             request = self._get_auth_refresh_token_request(
                 client,
@@ -403,9 +452,9 @@ class FiefAsync(BaseFief):
         )
 
     async def userinfo(self, access_token: str) -> Dict[str, Any]:
-        userinfo_endpoint = (await self._get_openid_configuration())[
-            "userinfo_endpoint"
-        ]
+        userinfo_endpoint = self._get_endpoint_url(
+            await self._get_openid_configuration(), "userinfo_endpoint"
+        )
         async with self._get_httpx_client() as client:
             request = self._get_userinfo_request(
                 client, endpoint=userinfo_endpoint, access_token=access_token
@@ -422,7 +471,11 @@ class FiefAsync(BaseFief):
 
     @contextlib.asynccontextmanager
     async def _get_httpx_client(self):
-        async with httpx.AsyncClient(base_url=self.base_url) as client:
+        headers = {}
+        if self.host is not None:
+            headers["Host"] = self.host
+
+        async with httpx.AsyncClient(base_url=self.base_url, headers=headers) as client:
             yield client
 
     async def _get_openid_configuration(self) -> Dict[str, Any]:
@@ -440,20 +493,27 @@ class FiefAsync(BaseFief):
         if self._jwks is not None:
             return self._jwks
 
-        jwks_uri = (await self._get_openid_configuration())["jwks_uri"]
+        jwks_uri = self._get_endpoint_url(
+            await self._get_openid_configuration(), "jwks_uri"
+        )
         async with self._get_httpx_client() as client:
             response = await client.get(jwks_uri)
             self._jwks = jwk.JWKSet.from_json(response.text)
             return self._jwks
 
-    async def _auth_exchange_token(self, code: str, redirect_uri) -> FiefTokenResponse:
-        token_endpoint = (await self._get_openid_configuration())["token_endpoint"]
+    async def _auth_exchange_token(
+        self, code: str, redirect_uri: str, *, code_verifier: Optional[str] = None
+    ) -> FiefTokenResponse:
+        token_endpoint = self._get_endpoint_url(
+            await self._get_openid_configuration(), "token_endpoint"
+        )
         async with self._get_httpx_client() as client:
             request = self._get_auth_exchange_token_request(
                 client,
                 endpoint=token_endpoint,
                 code=code,
                 redirect_uri=redirect_uri,
+                code_verifier=code_verifier,
             )
             response = await client.send(request)
 
